*************************************************************************************
*
* Miscellaneous functions
*
*************************************************************************************
#include "hconp000.ch"

FUNCTION l3w_fSetsys( aArgsin )
LOCAL cArg := "", cValue := "", lCreate := .F.
LOCAL cExepath := GetCurrentFolder()
LOCAL cIniName := 'system.ini'

PUBLIC l3w_aIni := {}
PUBLIC l3w_aMsg := {}

REQUEST HB_CODEPAGE_IT437
REQUEST HB_CODEPAGE_IT850
REQUEST HB_CODEPAGE_ITISO
REQUEST HB_CODEPAGE_ITISB

HB_LANGSELECT("IT")
HB_SETCODEPAGE( "ITISO" )

IF PCOUNT() == 1
 FOR EACH cArg IN aArgsIn
  IF .NOT. EMPTY(cArg)
   cValue := SUBSTR(cArg, 3)
   DO CASE
    CASE LOWER(SUBSTR(cArg, 1, 2)) == "-i"
     IF LEN(ALLTRIM(cValue)) > 0
      cIniName := cValue
     ENDIF
    CASE LOWER(SUBSTR(cArg, 1, 2)) == "-d"
     IF LEN(ALLTRIM(cValue)) > 0
      cExepath := cValue
     ENDIF
   ENDCASE
  ENDIF
 NEXT
ENDIF

SET( _SET_EPOCH, 2000 )
SET CENTURY ON
SET( _SET_DATEFORMAT, "dd-mm-yyyy" )
SET( _SET_PATH, cExepath )
SET( _SET_DEFAULT, cExepath )
SET( _SET_EXCLUSIVE, FALSE )
SET( _SET_DELETED, FALSE )

* ATTENTION: special treatment
AADD(l3w_aIni, {"application", "path",    cExepath})

* Application
l3w_fConf("Application", "Inifile",     cIniName)
l3w_fConf("Application", "Title",       "")
l3w_fConf("Application", "Name",        "")
l3w_fConf("Application", "Version",     "")
* Application fonts
l3w_fConf("ButtFont",    "Name",        "Arial")
l3w_fConf("ButtFont",    "Size",        "8")
l3w_fConf("FormFont",    "Name",        "Arial")
l3w_fConf("FormFont",    "Size",        "10")
* Application Messages File
l3w_fConf("Language",    "Langfile",    "")
* user
l3w_fConf("User",        "Name",        "Guest user")
l3w_fConf("User",        "Login",       "guest")
l3w_fConf("User",        "Password",    "")

lCreate := l3w_fOnInit()

RETURN lCreate

*==============================================================================*
* OnInit: .T. if ALL Ok else .F.
*==============================================================================*
FUNCTION l3w_fOnInit()
LOCAL cExePath := l3w_fConf("Application", "Path"), cIniName := l3w_fConf("Application", "Inifile")
LOCAL cFileIni := HB_FnameMerge(cExePath, cIniName), cFileLang := "", cCurrDbId := ""
LOCAL hIni, aSect, cSection, cKey
LOCAL lDone := .T.

* read and import configuration file
IF FILE(cFileIni)
 hIni := hb_IniRead( cFileIni )
 IF Empty( hIni )
  lDone := .F.
  MsgStop("Configuration file" + Hb_OsNewLine() +  cFileIni + Hb_OsNewLine() + "is empty")
 ELSE
  FOR EACH cSection IN hIni:Keys
   aSect := hIni[ cSection ]
   FOR EACH cKey IN aSect:Keys
    l3w_fConf(cSection, cKey, aSect[cKey])
   NEXT
  NEXT
 ENDIF
ELSE
 lDone := .F.
 MsgStop("Dont' exists configuration file" + Hb_OsNewLine() +  cFileIni)
ENDIF

* read and import language messages
IF lDone == .T.
 cFileLang := l3w_fConf("Language", "Langfile")
 IF !EMPTY(cFileLang)
  IF FILE(cFileLang)
   hIni := hb_IniRead( cFileLang )
   IF Empty( hIni )
    lDone := .F.
    MsgStop("Language file" + Hb_OsNewLine() +  cFileLang + Hb_OsNewLine() +  "is empty")
   ELSE
    FOR EACH cSection IN hIni:Keys
     aSect := hIni[ cSection ]
     FOR EACH cKey IN aSect:Keys
      l3w_fMsg(cSection, cKey, aSect[cKey])
     NEXT
    NEXT
   ENDIF
  ELSE
   lDone := .F.
   MsgStop("Dont' exists language file" + Hb_OsNewLine() +  cFileLang)
  ENDIF
 ENDIF
ENDIF

* set some values for Database
IF lDone == .T.
 l3w_fSetStdDbValue()
ENDIF
RETURN lDone

*==============================================================================*
* Conf
* to standardize the information, section and key will be transformed into lower value
* specal treatment for some values
* %exepath% wil be transformd into Project Path
*==============================================================================*
FUNCTION l3w_fConf( cSection, cKey, cValue )
LOCAL nX := ASCAN(l3w_aIni, {|x| x[1] == "application" .AND. x[2] == "path"})
LOCAL cExePath := l3w_aIni[nX,3]

IF PCOUNT() >= 2
 nX := ASCAN(l3w_aIni, {|x| x[1] == LOWER(cSection) .AND. x[2] == LOWER(cKey)})
ENDIF
IF PCOUNT() == 2
 IF nX <> 0
  RETURN l3w_aIni[nX,3]
 ELSE
*  RETURN NIL too dangerous
  RETURN ""
 ENDIF
ELSEIF PCOUNT() == 3
 cValue := IIF( AT("%exepath%", cValue) <> 0, STRTRAN(cValue, "%exepath%", cExePath), cValue )
 IF nX <> 0
  l3w_aIni[nX,3] := cValue
 ELSE
  AADD(l3w_aIni, {LOWER(cSection), LOWER(cKey), cValue})
 ENDIF
ENDIF

RETURN NIL

*==============================================================================*
* Msg
* to standardize the information, section and key will be transformed into lower value
*==============================================================================*
FUNCTION l3w_fMsg( cSection, cKey, cValue )
LOCAL nX := 0

IF PCOUNT() >= 2
 nX := ASCAN(l3w_aMsg, {|x| x[1] == LOWER(cSection) .AND. x[2] == LOWER(cKey)})
ENDIF
IF PCOUNT() == 2
 IF nX <> 0
  RETURN l3w_aMsg[nX,3]
 ELSE
*  RETURN NIL too dangerous
  RETURN "Undef"
 ENDIF
ELSEIF PCOUNT() == 3
 IF nX <> 0
  l3w_aMsg[nX,3] := cValue
 ELSE
  AADD(l3w_aMsg, {LOWER(cSection), LOWER(cKey), cValue})
 ENDIF
ENDIF

RETURN NIL

*==============================================================================*
* Database functions: useful for Dbf, MySql, PostgreSql
* - l3w_fDbUsed: return .T. if Database section exists
* - l3w_fSetStdDbValue : set [Database],[Current] and [Database],[Description] if Database section exist
* - l3w_fGetAllDbName : return an array with Database name if Database section exists
*==============================================================================*
FUNCTION l3w_fDbUsed()
LOCAL cDefault := l3w_fConf("Database", "Default")
LOCAL lReturn := IIF( VALTYPE(cDefault)=="C" .AND. LEN(ALLTRIM(cDefault))>0, .T., .F. )
RETURN lReturn

FUNCTION l3w_fSetStdDbValue()
LOCAL cDefault := l3w_fConf("Database", "Default")
LOCAL cDescr := l3w_fConf(cDefault, "Description")
IF l3w_fDbUsed()
 l3w_fConf("Database", "Current", cDefault)
 l3w_fConf("Database", "Description", cDescr)
ENDIF
RETURN NIL

FUNCTION l3w_fGetAllDbName()
LOCAL nX := 0, aReturn := {}
IF l3w_fDbUsed()
 FOR nX := 1 TO LEN(l3w_aIni)
  IF LOWER(l3w_aIni[nX,1]) == "database" .AND. AT("dbid", LOWER(l3w_aIni[nX,2])) <> 0
   AADD(aReturn, l3w_aIni[nX,3])
  ENDIF
 NEXT nX
ENDIF
RETURN aReturn

*==============================================================================*
* Return phisical file name with path of a DBF name
* M=Mandatory, O=optional
* Arg1 => (M) Table name
*==============================================================================*
FUNCTION l3w_fDbfWpath(cArg1)
LOCAL cPath := "", cFile := "", cExt := ""
LOCAL cReturn := ""
* first split cArg1 to ensure extract only a name without path, extension, ...
Hb_FnameSplit(cArg1, @cPath, @cFile, @cExt)
* find dbf path
cPath := l3w_fGetDbPath(cFile)
* merge datas
cReturn := Hb_FnameMerge(cPath, cFile,".dbf")
RETURN cReturn

*==============================================================================*
* Return phisical file name with path of a NTX name associated to a DBF name
* M=Mandatory, O=optional
* Arg1 => (M) NTX name
* Arg2 => (M) Table name
*==============================================================================*
FUNCTION l3w_fNtxWpath(cArg1, cArg2)
LOCAL cDbfPath := "", cNtxPath := "", cFile := "", cExt := ""
LOCAL cReturn := ""
* first split cArg2 to ensure extract only a name without path, extension, ...
Hb_FnameSplit(cArg2, @cDbfPath, @cFile, @cExt)
* find DBF path
cDbfPath := l3w_fGetDbPath(cFile)
* first split cArg1 to ensure extract only a name without path, extension, ...
Hb_FnameSplit(cArg1, @cNtxPath, @cFile, @cExt)
* merge datas
cReturn := Hb_FnameMerge(cDbfPath, cFile, ".ntx")
RETURN cReturn

*==============================================================================*
* Return Dbf path:
* M=Mandatory, O=optional
* Arg1 => (O) Table name
* if Arg1 it's empty, return current Db path (if filled) else current exe path
* else
* dbf path (if filled) else current Db path (if filled) else current exe path
*==============================================================================*
FUNCTION l3w_fGetDbPath(cArg1)
LOCAL cPath := "", cFile := "", cExt := ""
LOCAL cCurDbId := l3w_fConf("DataBase", "Current")
LOCAL cCommPath := l3w_fConf(cCurDbId, "CommonPath")
LOCAL cExePath := l3w_fConf("Application", "Path")
LOCAL cPathReturn := "", cDbfName := "", cSpecialPath := ""
* preset value
cPathReturn := IIF( LEN(ALLTRIM(cCommPath)) > 0, cCommPath, cExePath)
* check Arg1
IF PCOUNT()==1 .AND. VALTYPE(cArg1)=="C" .AND. LEN(ALLTRIM(cArg1))>0
* first split cArg1 to ensure extract only a name without path, extension, ...
 HB_FNameSplit(cArg1, @cPath, @cFile, @cExt)
 cDbfName := cFile
* if extracted
 IF LEN(ALLTRIM(cDbfName)) > 0
  cSpecialPath := l3w_fConf(cCurDbId, cDbfName)
  IF LEN(ALLTRIM(cSpecialPath)) > 0
   cPathReturn := cSpecialPath
  ENDIF
 ENDIF
ENDIF
* last check
IF Hb_DirExists(cPathReturn) == .F.
 MsgInfo("Path undefined" + Hb_OsNewLine(), "l3w_fGetdbpath")
ENDIF
RETURN cPathReturn

*==============================================================================*
* Try to open a .DBF file
* M=Mandatory, O=optional
* Arg1 => (M) Table name
* Arg2 => (O) Type Exclusive or Shared
* Arg3 => (O) Array of index
* Arg4 => (O) Alias: if a table it's already open, you can open with different alias
* Arg5 => (O) Path: path it's retrieved from configuration file; optionally, you can assign a different path
*==============================================================================*
FUNCTION l3w_fNetuse( cArg1, cArg2, aArg3, cArg4, cArg5)
LOCAL lOpen := .F.
LOCAL cDbfName := "", lShared := .T., aIndex := {}, cAlias := {}, cDbfPath := {}
LOCAL cRetPath := "", cRetFile := "", cRetExt := ""
LOCAL cDbfFile := ""
LOCAL nSeconds := NET_SECS
IF PCOUNT() >= 1
 IF VALTYPE(cArg1) == "C"
* first split cArg1 to ensure extract only a name without path, extension, ...
 Hb_FnameSplit(cArg1, @cRetPath, @cRetFile, @cRetExt)
 IF LEN(cRetFile) > 0
  cDbfName := LOWER(cRetFile)
  lShared := .T.
  aIndex := {}
  cAlias := cDbfName
  cDbfPath := l3w_fGetDbPath(cDbfName)
* shared or esclusive mode as argument
  IF PCOUNT() >= 2 .AND. VALTYPE(cArg2) == "C"
   lShared := IIF( UPPER(ALLTRIM(cArg2)) == "E", .F., .T. )
   ENDIF
  ENDIF
* array of index as argument
  IF PCOUNT() >= 3
   aIndex := IIF( VALTYPE(aArg3)=="A", aArg3, {} )
  ENDIF
* Alias as argument
  IF PCOUNT() >= 4 .AND. VALTYPE(cArg4) == "C" .AND. LEN(ALLTRIM(cArg4)) > 0
   cAlias := LOWER(ALLTRIM(cArg4))
  ENDIF
* Path as argument
  IF PCOUNT() >= 5 .AND. VALTYPE(cArg5) == "C" .AND. LEN(ALLTRIM(cArg5)) > 0
    cDbfPath := LOWER(ALLTRIM(cArg5))
  ENDIF
* set phisical file name
  cDbfFile := Hb_FnameMerge( cDbfPath, cDbfName, ".dbf")
* file exists ?
  IF FILE(cDbfFile)
* check if already open
   IF SELECT(cAlias) <= 0
    REPEAT
     DBUSEAREA( .T., DBSETDRIVER(), cDbfFile, cAlias, lShared, .F.)
     IF NETERR()
      nSeconds -= NET_WAIT
      INKEY( NET_WAIT)
     ELSE
      nSeconds := -1
      lOpen := .T.
     ENDIF
    UNTIL (nSeconds < 0)
* open or not ?
    IF lOpen == .F.
     MsgStop( cDbfName + Hb_OsNewLine() + "Not open", "l3w_fNetuse()" )
    ENDIF
   ELSE
    MsgStop( cDbfName + Hb_OsNewLine() + "Already open", "l3w_fNetuse()" )
   ENDIF
  ELSE
   MsgStop( cDbfFile + Hb_OsNewLine() + "don't exists", "l3w_fNetuse()" )
  ENDIF
 ELSE
  MsgStop( "Wrong file name", "l3w_fNetuse()" )
 ENDIF
ELSE
 MsgStop( "Invalid parameters", "l3w_fNetuse()" )
ENDIF
* if opened try to open indexes
IF lOpen .AND. LEN(aIndex) > 0
 lOpen := l3w_fNetidx( cAlias, aIndex, cDbfPath)
ENDIF
RETURN lOpen

*==============================================================================*
* Try to open .NTX file (M=Mandatory, O=optional)
* M=Mandatory, O=optional
* Arg1 => (M) Alias
* Arg2 => (M) Array of index names
* Arg3 => (M) path
*==============================================================================*
FUNCTION l3w_fNetidx(cAlias, aNtxName, cDbfPath)
LOCAL lOpenNtx := .T., cNtxName := "", cNtxFile := ""
LOCAL nSeconds := NET_SECS
LOCAL cRetPath := "", cRetFile := "", cRetExt := ""
IF SELECT(cAlias) > 0
 (cAlias)->(ORDLISTCLEAR())
 FOR nCounter := 1 TO LEN(aNtxName)
* first split name to ensure extract only a name without path, extension, ...
  Hb_FnameSplit(aNtxName[nCounter], @cRetPath, @cRetFile, @cRetExt)
  IF LEN(cRetFile) > 0
   cNtxName := LOWER(cRetFile)
* set phisical file name
   cNtxFile := Hb_FnameMerge( cDbfPath, cNtxName, INDEXEXT())
* check if it exists
   IF FILE(cNtxFile)
* try to open/add
    IF (cAlias)->(ORDLISTADD(cNtxFile)) == .F.
     lOpenNtx := .F.
     MsgStop( cNtxName + " order not open", "l3w_fNetidx()")
    ENDIF
   ELSE
    lOpenNtx := .F.
    MsgStop( cNtxFile + " don't exists", "l3w_fNetidx()")
   ENDIF
  ELSE
   lOpenNtx := .F.
   MsgStop( "Wrong index name", "l3w_fNetidx()")
  ENDIF
 NEXT nCounter
ELSE
 lOpenNtx := .F.
 MsgStop( cAlias + Hb_OsNewLine() + "Not open", "l3w_fNetidx()")
ENDIF
* if opened set to the first order
IF lOpenNtx
 (cAlias)->(ORDSETFOCUS(1))
ENDIF
RETURN lOpenNtx

*==============================================================================*
* Record Lock
* Parametri passati
* cAliasName => alias tabella dbf. Se non  passato assume WorkArea corrente
* nRecNum => numero record deve essere >0 e < LASTREC. Se non  passato record corrente
* Parametri ritornati
* .T. se riuscito altrimenti .F.
*==============================================================================*
FUNCTION l3w_fRecLock(cAliasName, nRecNum)
LOCAL lLock := .F.
LOCAL cAlias := IIF(VALTYPE(cAliasName)=="C", ALLTRIM(cAliasName), "")
LOCAL nSeconds := NET_SECS * 2
LOCAL nCurArea := SELECT(), nArea := 0
LOCAL nNumRec := 0
nArea := IIF(LEN(cAlias)>0, SELECT(cAlias), SELECT())
IF nArea<>0
 DBSELECTAREA(nArea)
 IF VALTYPE(nRecNum)=="N" .AND. nRecNum>0 .AND. nRecNum<=LASTREC()
  nNumRec := nRecNum
 ELSE
  nNumrec := RECNO()
 ENDIF
 IF DBRLOCK(nNumRec)
  lLock := .T.
 ELSE
  WAIT WINDOW "Record Lock Processing..."  NOWAIT
  WHILE nSeconds>0
   DO EVENTS
   IF DBRLOCK(nNumRec)
    lLock := .T.
    EXIT
   ELSE
* waiting
    INKEY(NET_WAIT)
    nSeconds -= NET_WAIT
   ENDIF
  END
  WAIT CLEAR
 ENDIF
ENDIF
DBSELECTAREA(nCurArea)
RETURN lLock

*==============================================================================*
* Record Locked List
* Parametri passati
* cAliasName => alias tabella dbf. Se non  passato assume WorkArea corrente
* Parametri ritornati
* aRecList
*==============================================================================*
FUNCTION l3w_fRecLockList(cAliasName)
LOCAL cAlias := IIF(VALTYPE(cAliasName)=="C", ALLTRIM(cAliasName), "")
LOCAL nCurArea := SELECT(), nArea := 0
LOCAL aRecList := {}
nArea := IIF(LEN(cAlias)>0, SELECT(cAlias), SELECT())
IF nArea<>0
 DBSELECTAREA(nArea)
 aRecList := DBRLOCKLIST()
ENDIF
DBSELECTAREA(nCurArea)
RETURN aRecList

*==============================================================================*
* Record UnLock
* Parametri passati
* cAliasName => alias tabella dbf. Se non  passato assume WorkArea corrente
* nRecNum => numero record deve essere >0 e < LASTREC. Se non  passato record corrente
* Parametri ritornati
* .T. se riuscito altrimenti .F.
*==============================================================================*
FUNCTION l3w_fRecUnlock(cAliasName, nRecNum)
LOCAL lUnLock := .F.
LOCAL cAlias := IIF(VALTYPE(cAliasName)=="C", ALLTRIM(cAliasName), "")
LOCAL nSeconds := NET_SECS * 2
LOCAL nCurArea := SELECT(), nArea := 0
LOCAL nNumRec := 0
nArea := IIF(LEN(cAlias)>0, SELECT(cAlias), SELECT())
IF nArea<>0
 DBSELECTAREA(nArea)
 IF VALTYPE(nRecNum)=="N" .AND. nRecNum>0 .AND. nRecNum<=LASTREC()
  nNumRec := nRecNum
 ELSE
  nNumrec := RECNO()
 ENDIF
 DBRUNLOCK(nNumRec)
 lUnlock := .T.
ENDIF
DBSELECTAREA(nCurArea)
RETURN lUnlock

*==============================================================================*
* Add Record
* Parametri passati
* cAliasName => alias tabella dbf. Se non  passato assume WorkArea corrente
* lReleaseRecLocks => if  true (.T.),
*     clears all pending record locks, then appends the next record.  If
*     <lReleaseRecLocks> is false (.F.), all pending record locks are
*     maintained and the new record is added to the end of the Lock List.  The
*     default value of <lReleaseRecLocks> is true (.T.)
* Parametri ritornati
* .T. se riuscito altrimenti .F.
*==============================================================================*
FUNCTION l3w_fAddrec(cAliasName, lReleaseRecLocks)
LOCAL lAddRec := .F.
LOCAL cAlias := IIF(VALTYPE(cAliasName)=="C", ALLTRIM(cAliasName), "")
LOCAL lRelease := IIF(VALTYPE(lReleaseRecLocks)=="L", lReleaseRecLocks, .T.)
LOCAL nSeconds := NET_SECS * 2
LOCAL nCurArea := SELECT(), nArea := 0
nArea := IIF(LEN(cAlias)>0, SELECT(cAlias), SELECT())
IF nArea<>0
 DBSELECTAREA(nArea)
 DBAPPEND(lRelease)
 IF !NETERR()
  lAddRec := .T.
 ELSE
  WAIT WINDOW "Record Add Processing..."  NOWAIT
  WHILE nSeconds>0
   DO EVENTS
   DBAPPEND(lRelease)
   IF !NETERR()
    lAddRec := .T.
    EXIT
   ELSE
* waiting
    INKEY(NET_WAIT)
    nSeconds -= NET_WAIT
   ENDIF
  END
  WAIT CLEAR
 ENDIF
ENDIF
DBSELECTAREA(nCurArea)
RETURN lAddRec

*==============================================================================*
* fill out a key by nLen with cChar
*==============================================================================*
FUNCTION l3w_fFiller(cString, nLen, cChar)
LOCAL xString := IIF(VALTYPE(cString)=="C", cString, "")
LOCAL xLen := IIF(VALTYPE(nLen)=="N", nLen, LEN(xString))
LOCAL xChar := IIF(VALTYPE(cChar)=="C", cChar, SPACE(1))
LOCAL xReturn := ""
xReturn := SUBSTR(xString+REPLICATE(xChar,xLen), 1, xLen)
RETURN xReturn

*==============================================================================*
* this function return an array with controls defined in a form (many thanks to Rathinagiri)
* M=Mandatory, O=optional
* Arg1 (M) form name as character
*==============================================================================*
FUNCTION l3w_fGetWindowControls(cForm)
LOCAL aControlList := {}, nX := 0
IF VALTYPE(cForm)=="C"
 FOR nX := 1 TO LEN(_HMG_SYSDATA[4])
  IF _HMG_SYSDATA[4,nX] == GetFormHandle(cForm) .AND. ;
   VALTYPE(_HMG_SYSDATA [2,nX]) == "C" .AND. LEN(ALLTRIM(_HMG_SYSDATA [2,nX])) > 0 // aggiunto questa parte di test
   AADD(aControlList,_HMG_SYSDATA [2,nX])
  ENDIF
 NEXT nX
ENDIF
RETURN ACLONE(aControlList)

*==============================================================================*
* this function set one property for a list of a defined controls in a form
* SetProperty ( "nomeform" , 'nomecontrol' , 'Visible' , .f. )
* SetProperty ( "nomeform" , 'nomecontrol' , 'Enabled' , .f. )
* M=Mandatory, O=optional
* Arg1 (M) form name as character
* Arg2 (M) .T. or .f.
* Arg3 (M) array of defined controls
* Arg4 (M) 'Enabled' (defualt), Visible, ...
*==============================================================================*
FUNCTION l3w_fSetAllControls(cForm, lValue, aList, cType)
LOCAL aControlList := l3w_fGetWindowControls(cForm), nI := 0, nJ := 0
DEFAULT lValue := .T.
DEFAULT cType := 'Enabled'
IF VALTYPE(aList)=="A"
 FOR nI := 1 TO LEN(aList)
  nJ := ASCAN(aControlList, aList[nI])
  IF nJ <> 0
   SetProperty( cForm , aList[nI] , cType , lValue )
  ENDIF
 NEXT nI
ENDIF
RETURN NIL

*==============================================================================*
* check status form (form status code are indipendent from this function: take a look in program)
* N = form free for default
* I = form committed for default
*==============================================================================*
FUNCTION l3w_fFomFree( cForm, cControl, cArg3 )
LOCAL lFree := .F.
LOCAL cStatus := IIF(VALTYPE(cArg3)=="C", cArg3, "N")
IF _IsControlDefined(cControl, cForm)
 IF GetProperty( cForm, cControl, "Value" ) == cStatus
  lFree := .T.
 ENDIF
ENDIF
IF lFree==.F.
 MsgStop("Form " + cForm + " committed")
ENDIF
RETURN lFree
*==============================================================================*
*  check email
*==============================================================================*
FUNCTION l3w_fIsValidMail(cMail)
LOCAL lValid := .F., nAt := 0, cLocalPart := "", cDomain := ""
LOCAL lLocalPart := .T., cCharLocPart := "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ09123456789&'*+-/=?^_{}~."
LOCAL lDomain := .T., cCharDom := "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ09123456789.-"
IF VALTYPE(cMail)=="C"
 nAt := AT("@", cMail)
 IF nAt <> 0
  cLocalPart := SUBSTR(cMail, 1, nAt-1)
  cDomain := SUBSTR(cMail, nAt+1)
* NO other @, local_part max 64 chars, domain max 256
  IF AT("@", cLocalPart)==0 .AND. AT("@", cDomain)==0 .AND. ;
   LEN(cLocalPart)>=1 .AND. LEN(cLocalPart)<=64 .AND. ;
   LEN(cDomain)>=1 .AND. LEN(cDomain)<=256
* LOCAL_PART: check allowed characters
   FOR nAt := 1 TO LEN(cLocalPart)
    lLocalPart := IIF( AT(SUBSTR(cLocalPart, nAt, 1), cCharLocPart)==0, .F., lLocalPart )
   NEXT nAt
* LOCAL_PART: dot (.) can not be the first or the last character
   IF LEFT(cLocalPart, 1)=="." .OR. RIGHT(cLocalPart, 1)=="."
    lLocalPart := .F.
   ENDIF
* LOCAL_PART: double dot (..) not allowed
   IF AT("..", cLocalPart)<>0
    lLocalPart := .F.
   ENDIF
* DOMAIN: check allowed characters
   FOR nAt := 1 TO LEN(cDomain)
    lDomain := IIF( AT(SUBSTR(cDomain, nAt, 1), cCharDom)==0, .F., lDomain )
   NEXT nAt
* DOMAIN: dot (.) can not be the first or the last character
   IF LEFT(cDomain, 1)=="." .OR. RIGHT(cDomain, 1)=="."
    lLocalPart := .F.
   ENDIF
* DOMAIN: double dot (..) not allowed
   IF AT( "..", cDomain ) <> 0
    lLocalPart:= .F.
   ENDIF
* both LOCAL_PART and DOMAIN must be valid
   lValid := IIF( lLocalPart == .T. .AND. lDomain == .T., .T., .F. )
  ENDIF
 ENDIF
ENDIF
RETURN lValid

*==============================================================================*
*  split email
*==============================================================================*
FUNCTION l3w_fSplitMail(cMailaddr)
LOCAL aParts := {}
IF l3w_fIsValidMail(cMailaddr) == .T.
 aParts := Hb_RegexSplit("@", cMailaddr,,,1)
ENDIF
RETURN aParts

*==============================================================================*
* check permission user
* TODO
*==============================================================================*
FUNCTION l3w_fGetperms(cProcedure, cRequested)
LOCAL lGrant := .T.
RETURN lGrant

*==============================================================================*
* login a user
* TODO
*==============================================================================*
FUNCTION l3w_fLogin()
LOCAL lGrant := .T.
RETURN lGrant
